@@ -1,9 +1,41 @@
-use super::{TranspileReturn, Transpiler};
+/*
+ * Databind - Expand the functionality of Minecraft Datapacks.
+ * Copyright (C) 2021  Adam Thompson-Sharpe
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ */
+use super::Transpiler;
 
 use crate::token::Token;
 use rand::{distributions::Alphanumeric, Rng};
 use std::collections::HashMap;
 
+/// Return from the transpiler
+///
+/// # Arguments
+///
+/// - `file_contents` - A list of file contents
+/// - `filename_map` - A map of filenames to indexes in the file_contents Vec
+/// - `var_map` - A map of variable names used in files to randomized names
+/// - `tag_map` - A map of tags to functions
+pub struct TranspileReturn {
+    pub file_contents: Vec<String>,
+    pub filename_map: HashMap<String, usize>,
+    pub var_map: HashMap<String, String>,
+    pub tag_map: HashMap<String, Vec<String>>,
+}
+
 impl Transpiler<'_> {
     /// Convert tokens to a transpiled file or files
     ///
@@ -12,27 +44,24 @@ impl Transpiler<'_> {
     /// - `tokens` - A list of tokens
     /// - `namespace` - The namespace to use for functions, if relevant
     /// - `existing_var_map` - An existing map of variables to randomized names
-    /// - `return_var_map` - Whether to return the var map used
-    /// - `return_multi_file` - Whether to return multiple files
     pub fn transpile(
         &self,
         tokens: Vec<Token>,
         namespace: Option<&str>,
         existing_var_map: Option<&HashMap<String, String>>,
-        return_var_map: bool,
-        return_multi_file: bool,
     ) -> TranspileReturn {
-        // let mut transpiled = String::new();
+        let tokens = self.while_convert(tokens);
+
         let mut var_map: HashMap<String, String>;
+        let mut tag_map: HashMap<String, Vec<String>> = HashMap::new();
 
         // A vector of file contents
         let mut files: Vec<String> = vec![String::new()];
         // A map of filenames to indexes in the files vector
         let mut filename_to_index: HashMap<String, usize> = HashMap::new();
-        filename_to_index.insert(String::new(), 0);
 
-        if let Some(_) = existing_var_map {
-            var_map = existing_var_map.unwrap().clone();
+        if let Some(map) = existing_var_map {
+            var_map = map.clone();
         } else {
             var_map = HashMap::new();
         }
@@ -43,14 +72,24 @@ impl Transpiler<'_> {
         let mut current_var = String::new();
         let mut assignment_operator = Token::None;
         // For functions
-        let mut in_function = false;
-        let mut current_function = String::new();
+        let mut func_depth = 0;
+        let mut current_functions: Vec<String> = Vec::new();
         let mut calling_function = false;
         // For objective-related tokens
         let mut current_objective = String::new();
         let mut objective_target = String::new();
 
         for token in tokens.iter() {
+            // Don't transpile contents outside of a function
+            match token {
+                Token::DefineFunc => {}
+                _ => {
+                    if func_depth == 0 {
+                        continue;
+                    }
+                }
+            }
+
             match token {
                 Token::Var => active_token = Token::Var,
                 Token::TestVar => active_token = Token::TestVar,
@@ -64,10 +103,10 @@ impl Transpiler<'_> {
                             if var_map.contains_key(var) {
                                 let to_add = format!("score --databind {} ", var_map[var]);
 
-                                if !in_function {
+                                if func_depth == 0 {
                                     files[0].push_str(&to_add[..]);
                                 } else {
-                                    files[filename_to_index[&current_function]]
+                                    files[filename_to_index[&current_functions[func_depth - 1]]]
                                         .push_str(&to_add[..]);
                                 }
                             } else {
@@ -76,10 +115,11 @@ impl Transpiler<'_> {
                             }
                         } else {
                             let to_add = format!("score --databind {} ", var);
-                            if !in_function {
+                            if func_depth == 0 {
                                 files[0].push_str(&to_add[..]);
                             } else {
-                                files[filename_to_index[&current_function]].push_str(&to_add[..]);
+                                files[filename_to_index[&current_functions[func_depth - 1]]]
+                                    .push_str(&to_add[..]);
                             }
                         }
                     }
@@ -93,7 +133,7 @@ impl Transpiler<'_> {
                     if !calling_function {
                         files.push(String::new());
                         filename_to_index.insert(name.clone(), files.len() - 1);
-                        current_function = name.clone();
+                        current_functions.push(name.clone());
                     // Function call
                     } else {
                         // Function contains namespace
@@ -101,32 +141,44 @@ impl Transpiler<'_> {
 
                         if has_namespace {
                             let to_add = format!("function {}", name);
-                            if !in_function {
+                            if func_depth == 0 {
                                 files[0].push_str(&to_add[..]);
                             } else {
-                                files[filename_to_index[&current_function]].push_str(&to_add[..]);
+                                files[filename_to_index[&current_functions[func_depth - 1]]]
+                                    .push_str(&to_add[..]);
+                            }
+                        } else if let Some(ns) = namespace {
+                            let to_add = format!("function {}:{}", ns, name);
+                            if func_depth == 0 {
+                                files[0].push_str(&to_add[..]);
+                            } else {
+                                files[filename_to_index[&current_functions[func_depth - 1]]]
+                                    .push_str(&to_add[..]);
                             }
                         } else {
-                            if let Some(ns) = namespace {
-                                let to_add = format!("function {}:{}", ns, name);
-                                if !in_function {
-                                    files[0].push_str(&to_add[..]);
-                                } else {
-                                    files[filename_to_index[&current_function]]
-                                        .push_str(&to_add[..]);
-                                }
-                            } else {
-                                println!("No namespace provided for function call.");
-                                std::process::exit(2);
-                            }
+                            panic!("No namespace provided for function call.");
                         }
 
                         calling_function = false;
                     }
                 }
                 Token::CallFunc => calling_function = true,
-                Token::DefineFunc => in_function = true,
-                Token::EndFunc => in_function = false,
+                Token::DefineFunc => func_depth += 1,
+                Token::EndFunc => {
+                    func_depth -= 1;
+                    current_functions.pop();
+                }
+                Token::TagName(tag) => {
+                    if func_depth == 0 {
+                        println!("Tag found outside of function.");
+                        std::process::exit(1);
+                    }
+
+                    tag_map
+                        .entry(tag.clone())
+                        .or_insert_with(Vec::new)
+                        .push(current_functions[func_depth - 1].clone());
+                }
                 Token::ObjectiveName(name) => current_objective = name.clone(),
                 // An objective type will always be the last part of a new objective
                 Token::ObjectiveType(objective) => {
@@ -148,10 +200,10 @@ impl Transpiler<'_> {
                                     var_map[&current_objective], objective, current_objective
                                 );
 
-                                if !in_function {
+                                if func_depth == 0 {
                                     files[0].push_str(&to_add[..]);
                                 } else {
-                                    files[filename_to_index[&current_function]]
+                                    files[filename_to_index[&current_functions[func_depth - 1]]]
                                         .push_str(&to_add[..]);
                                 }
                             } else {
@@ -160,10 +212,10 @@ impl Transpiler<'_> {
                                     var_map[&current_objective], objective
                                 );
 
-                                if !in_function {
+                                if func_depth == 0 {
                                     files[0].push_str(&to_add[..]);
                                 } else {
-                                    files[filename_to_index[&current_function]]
+                                    files[filename_to_index[&current_functions[func_depth - 1]]]
                                         .push_str(&to_add[..]);
                                 }
                             }
@@ -177,10 +229,11 @@ impl Transpiler<'_> {
                             current_objective, objective
                         );
 
-                        if !in_function {
+                        if func_depth == 0 {
                             files[0].push_str(&to_add[..]);
                         } else {
-                            files[filename_to_index[&current_function]].push_str(&to_add[..]);
+                            files[filename_to_index[&current_functions[func_depth - 1]]]
+                                .push_str(&to_add[..]);
                         }
                     }
                     active_token = Token::None;
@@ -208,10 +261,11 @@ impl Transpiler<'_> {
                                                 var_map[&current_var], current_var
                                             );
 
-                                            if !in_function {
+                                            if func_depth == 0 {
                                                 files[0].push_str(&to_add[..]);
                                             } else {
-                                                files[filename_to_index[&current_function]]
+                                                files[filename_to_index
+                                                    [&current_functions[func_depth - 1]]]
                                                     .push_str(&to_add[..]);
                                             }
                                         } else {
@@ -220,10 +274,11 @@ impl Transpiler<'_> {
                                                 var_map[&current_var]
                                             );
 
-                                            if !in_function {
+                                            if func_depth == 0 {
                                                 files[0].push_str(&to_add[..]);
                                             } else {
-                                                files[filename_to_index[&current_function]]
+                                                files[filename_to_index
+                                                    [&current_functions[func_depth - 1]]]
                                                     .push_str(&to_add[..]);
                                             }
                                         }
@@ -232,10 +287,11 @@ impl Transpiler<'_> {
                                             var_map[&current_var], int
                                         );
 
-                                        if !in_function {
+                                        if func_depth == 0 {
                                             files[0].push_str(&to_add[..]);
                                         } else {
-                                            files[filename_to_index[&current_function]]
+                                            files[filename_to_index
+                                                [&current_functions[func_depth - 1]]]
                                                 .push_str(&to_add[..]);
                                         }
                                     } else {
@@ -250,10 +306,11 @@ impl Transpiler<'_> {
                                         current_var
                                     );
 
-                                    if !in_function {
+                                    if func_depth == 0 {
                                         files[0].push_str(&to_add[..]);
                                     } else {
-                                        files[filename_to_index[&current_function]]
+                                        files
+                                            [filename_to_index[&current_functions[func_depth - 1]]]
                                             .push_str(&to_add[..]);
                                     }
 
@@ -262,10 +319,11 @@ impl Transpiler<'_> {
                                         current_var, int
                                     );
 
-                                    if !in_function {
+                                    if func_depth == 0 {
                                         files[0].push_str(&to_add[..]);
                                     } else {
-                                        files[filename_to_index[&current_function]]
+                                        files
+                                            [filename_to_index[&current_functions[func_depth - 1]]]
                                             .push_str(&to_add[..]);
                                     }
                                 }
@@ -284,10 +342,11 @@ impl Transpiler<'_> {
                                             action, var_map[&current_var], int
                                         );
 
-                                        if !in_function {
+                                        if func_depth == 0 {
                                             files[0].push_str(&to_add[..]);
                                         } else {
-                                            files[filename_to_index[&current_function]]
+                                            files[filename_to_index
+                                                [&current_functions[func_depth - 1]]]
                                                 .push_str(&to_add[..]);
                                         }
                                     } else {
@@ -303,10 +362,11 @@ impl Transpiler<'_> {
                                         action, &current_var, int
                                     );
 
-                                    if !in_function {
+                                    if func_depth == 0 {
                                         files[0].push_str(&to_add[..]);
                                     } else {
-                                        files[filename_to_index[&current_function]]
+                                        files
+                                            [filename_to_index[&current_functions[func_depth - 1]]]
                                             .push_str(&to_add[..]);
                                     }
                                 }
@@ -332,10 +392,11 @@ impl Transpiler<'_> {
                                             int
                                         );
 
-                                        if !in_function {
+                                        if func_depth == 0 {
                                             files[0].push_str(&to_add[..]);
                                         } else {
-                                            files[filename_to_index[&current_function]]
+                                            files[filename_to_index
+                                                [&current_functions[func_depth - 1]]]
                                                 .push_str(&to_add[..]);
                                         }
                                     } else {
@@ -351,17 +412,17 @@ impl Transpiler<'_> {
                                         action, objective_target, &current_objective, int
                                     );
 
-                                    if !in_function {
+                                    if func_depth == 0 {
                                         files[0].push_str(&to_add[..]);
                                     } else {
-                                        files[filename_to_index[&current_function]]
+                                        files
+                                            [filename_to_index[&current_functions[func_depth - 1]]]
                                             .push_str(&to_add[..]);
                                     }
                                 }
                             }
                             _ => {
-                                println!(".= operator was mistakenly tokenized for objective");
-                                std::process::exit(2);
+                                panic!(".= operator was tokenized for objective");
                             }
                         }
                     }
@@ -369,36 +430,35 @@ impl Transpiler<'_> {
                     assignment_operator = Token::None;
                 }
                 Token::NonDatabind(string) => {
-                    if !in_function {
+                    if func_depth == 0 {
                         files[0].push_str(string);
                     } else {
-                        files[filename_to_index[&current_function]].push_str(string);
+                        files[filename_to_index[&current_functions[func_depth - 1]]]
+                            .push_str(string);
                     }
                 }
                 Token::NewLine => {
-                    if !in_function {
+                    if func_depth == 0 {
                         files[0].push('\n');
                     } else {
-                        files[filename_to_index[&current_function]].push('\n');
+                        files[filename_to_index[&current_functions[func_depth - 1]]].push('\n');
                     }
                 }
                 _ => {}
             }
         }
 
-        // Remove leading/trailing whitespace from files
+        // Remove leading/trailing whitespace from files as well as empty lines
         for file in files.iter_mut() {
             *file = file.trim().to_string();
+            *file = file.replace("\n\n", "\n");
         }
 
-        if return_var_map && return_multi_file {
-            TranspileReturn::MultiFileAndMap(files, filename_to_index, var_map)
-        } else if !return_var_map && !return_multi_file {
-            TranspileReturn::SingleContents(files[0].clone())
-        } else if return_var_map && !return_multi_file {
-            TranspileReturn::SingleContentsAndMap(files[0].clone(), var_map)
-        } else {
-            TranspileReturn::MultiFile(files, filename_to_index)
+        TranspileReturn {
+            file_contents: files,
+            filename_map: filename_to_index,
+            var_map,
+            tag_map,
         }
     }
 }
